@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * As an exception, when this program is distributed through (i) the App Store
  * by Apple Inc.; (ii) the Mac App Store by Apple Inc.; or (iii) Google Play
@@ -31,6 +31,7 @@
 #include <iostream>
 #include <string>
 #include <cmath>
+#include <cassert>
 
 #include <libnormaliz/general.h>
 #include <libnormaliz/integer.h>
@@ -70,10 +71,10 @@ class Matrix {
     //---------------------------------------------------------------------------
     //              Row and column reduction
     //---------------------------------------------------------------------------
-    // return value false undicates failure because of overflow
+    // return value false indicates failure because of overflow
     // for all the routines below
 
-    // reduction via integer division and elemntary transformations
+    // reduction via integer division and elementary transformations
     bool reduce_row(size_t corner);           // reduction by the corner-th row
     bool reduce_row(size_t row, size_t col);  // corner at position (row,col)
 
@@ -95,7 +96,7 @@ class Matrix {
     // solution replaces right hand side
     bool solve_destructive_inner(bool ZZinvertible, Integer& denom);
 
-    // asembles the matrix of the system (left side the submatrix of mother given by key
+    // assembles the matrix of the system (left side the submatrix of mother given by key
     // right side from column vectors pointed to by RS
     // both in a single matrix
     void solve_system_submatrix_outer(const Matrix<Integer>& mother,
@@ -127,7 +128,7 @@ class Matrix {
     vector<key_t> max_rank_submatrix_lex_inner(bool& success, vector<key_t> perm = vector<key_t>(0)) const;
 
     // A version of invert that circumvents protection and leaves it to the calling routine
-    Matrix invert_unprotected(Integer& denom, bool& sucess) const;
+    Matrix invert_unprotected(Integer& denom, bool& success) const;
 
     bool SmithNormalForm_inner(size_t& rk, Matrix<Integer>& Right);
 
@@ -161,6 +162,9 @@ class Matrix {
 
     size_t row_echelon_inner_bareiss(bool& success, Integer& det);
 
+    // makes entries above dial  <= 0
+    bool reduce_rows_upwards_negative();
+
     vector<vector<Integer>*> submatrix_pointers(const vector<key_t>& key);
 
     //---------------------------------------------------------------------------
@@ -169,6 +173,10 @@ class Matrix {
 
     void exchange_rows(const size_t& row1, const size_t& row2);     // row1 is exchanged with row2
     void exchange_columns(const size_t& col1, const size_t& col2);  // col1 is exchanged with col2
+    void cyclic_shift_right(const size_t& col); // involving columns 0,...,col
+    void cyclic_shift_left(const size_t& col); // ditto, but opposite
+    void permute_columns(const vector<key_t>& perm); // column perm[i] becomes colimn i
+    void inverse_permute_columns(const vector<key_t>& perm); // the inverse operation
 
     //---------------------------------------------------------------------------
 
@@ -180,7 +188,7 @@ class Matrix {
     Matrix(size_t dim);                             // constructor of unit matrix
     Matrix(size_t row, size_t col);                 // main constructor, all entries 0
     Matrix(size_t row, size_t col, Integer value);  // constructor, all entries set to value
-    Matrix(const vector<vector<Integer> >& elem);   // constuctor, elem=elem
+    Matrix(const vector<vector<Integer> >& elem);   // constructor, elem=elem
     Matrix(const list<vector<Integer> >& elems);
     Matrix(const vector<Integer>& row);
 
@@ -192,8 +200,11 @@ class Matrix {
     void print(const string& name, const string& suffix) const;         //  writes matrix into name.suffix
     void print_append(const string& name, const string& suffix) const;  // the same, but appends matrix
     void print(std::ostream& out, bool with_format = true) const;       // writes matrix to the stream
-    void pretty_print(std::ostream& out, bool with_row_nr = false, bool count_from_one=false)
-        const;                     // writes matrix in a nice format to the stream                   // read a row
+    void sparse_print(const string& name, const string& suffix) const;         //  writes matrix into name.suffix
+    void sparse_print(std::ostream& out, bool with_format = true) const;    // writes matrix sparse to the stream
+    void debug_print(char mark = '*') const;
+    void pretty_print(std::ostream& out, bool with_row_nr = false, bool count_from_one = false)
+        const;                     // writes matrix in a nice format to the stream
     size_t nr_of_rows() const;     // returns nr
     size_t nr_of_columns() const;  // returns nc
     void set_nr_of_columns(size_t c);
@@ -235,14 +246,15 @@ class Matrix {
     void resize_columns(size_t nr_cols);
     void Shrink_nr_rows(size_t new_nr_rows);
 
-    vector<Integer> diagonal() const;  // returns the diagonale of this
-                                       // this should be a quadratic matrix
+    vector<Integer> diagonal() const;       // returns the diagonal of this
+                                            // this should be a quadratic matrix
     size_t maximal_decimal_length() const;  // return the maximal number of decimals
                                             // needed to write an entry
 
     vector<size_t> maximal_decimal_length_columnwise() const;  // the same per column
 
-    void append(const Matrix& M);                    // appends the rows of M to this
+    void append(const Matrix& M); // appends the rows of M to this
+    void swap_append(Matrix& M); // ditto, but lines are swapped
     void append(const vector<vector<Integer> >& M);  // the same, but for another type of matrix
     void append(const vector<Integer>& v);           // append the row v to this
     void append_column(const vector<Integer>& v);    // append the column v to this
@@ -277,6 +289,8 @@ class Matrix {
         nc = rows;
     }
 
+    size_t mult_of_eigenvalue(const Integer& ev);
+
     //  convert the remaining matrix to nmz_float
     Matrix<nmz_float> nmz_float_without_first_column() const;
 
@@ -301,7 +315,7 @@ class Matrix {
     // bool equal(const Matrix& A, long m) const;     // returns this==A (mod m)
     Matrix transpose() const;  // returns the transpose of this
     void transpose_in_place();
-    
+
     bool zero_product_with_transpose_of(const Matrix& B);
 
     bool is_diagonal() const;
@@ -316,7 +330,7 @@ class Matrix {
     void reduction_modulo(const Integer& modulo);  // this=this mod scalar
     Integer matrix_gcd() const;                    // returns the gcd of all elem
     vector<Integer> make_prime();                  // each row of this is reduced by its gcd,
-                                   // vector of gcds returned
+                                                   // vector of gcds returned
     void make_cols_prime(size_t from_col, size_t to_col);
     // the columns of this in the specified range are reduced by their gcd
     void simplify_rows(const vector<Integer>& Norm);  // applies v_standardize to the rows
@@ -364,7 +378,7 @@ class Matrix {
 
     size_t rank() const;                                    // returns rank
     Integer full_rank_index() const;                        // returns index of full rank sublattice
-    size_t rank_submatrix(const vector<key_t>& key) const;  // returns rank of submarix defined by key
+    size_t rank_submatrix(const vector<key_t>& key) const;  // returns rank of submatrix defined by key
 
     // returns rank of submatrix of mother. "this" is used as work space
     size_t rank_submatrix(const Matrix<Integer>& mother, const vector<key_t>& key);
@@ -374,7 +388,7 @@ class Matrix {
     Integer vol_submatrix(const vector<key_t>& key) const;
     Integer vol_submatrix(const Matrix<Integer>& mother, const vector<key_t>& key);
 
-    // find linearly indepenpendent submatrix of maximal rank
+    // find linearly independent submatrix of maximal rank
 
     vector<key_t> max_rank_submatrix_lex(vector<key_t> perm = vector<key_t>(0)) const;  // returns a vector with entries
     // the indices of the first rows in lexicographic order of this forming
@@ -385,7 +399,7 @@ class Matrix {
     // In the following routines, denom is the absolute value of the determinant of the
     // left side matrix.
     // If the diagonal is asked for, ZZ-invertible transformations are used.
-    // Otherwise ther is no restriction on the used algorithm
+    // Otherwise there is no restriction on the used algorithm
 
     // The diagonal of left hand side after transformation into an upper triangular matrix
     // is saved in diagonal, denom is |determinant|.
@@ -435,7 +449,7 @@ class Matrix {
     // computes solution vector for right side v
     // insists on integrality of the solution
 
-    // homogenous linear systems
+    // homogeneous linear systems
 
     Matrix<Integer> kernel(bool use_LLL = true) const;
     // computes a ZZ-basis of the solutions of (*this)x=0
@@ -453,15 +467,19 @@ class Matrix {
                           bool make_sol_prime = false) const;
 
     // the sdame with prefabricated work matrices
-    void invert_submatrix(const vector<key_t>& key, Integer& denom, Matrix<Integer>& Inv, 
-                          Matrix<Integer>& Work, Matrix<Integer>& unitMat, 
-                          bool compute_denom = true, bool make_sol_prime = false) const;
+    void invert_submatrix(const vector<key_t>& key,
+                          Integer& denom,
+                          Matrix<Integer>& Inv,
+                          Matrix<Integer>& Work,
+                          Matrix<Integer>& unitMat,
+                          bool compute_denom = true,
+                          bool make_sol_prime = false) const;
 
     // find linear form that is constant on the rows
 
     vector<Integer> find_linear_form() const;
     // Tries to find a linear form which gives the same value an all rows of this
-    // this should be a m x n matrix (m>=n) of maxinal rank
+    // this should be a m x n matrix (m>=n) of maximal rank
     // returns an empty vector if there does not exist such a linear form
 
     vector<Integer> find_linear_form_low_dim() const;
@@ -482,8 +500,12 @@ class Matrix {
 
     // computes support hyperplanes and volume, second version with prefabricated work matrices
     void simplex_data(const vector<key_t>& key, Matrix<Integer>& Supp, Integer& vol, bool compute_vol) const;
-    void simplex_data(const vector<key_t>& key, Matrix<Integer>& Supp, Integer& vol,Matrix<Integer>& Work,
-                      Matrix<Integer>& unitMat, bool compute_vol) const;
+    void simplex_data(const vector<key_t>& key,
+                      Matrix<Integer>& Supp,
+                      Integer& vol,
+                      Matrix<Integer>& Work,
+                      Matrix<Integer>& unitMat,
+                      bool compute_vol) const;
 
     // finds subdivision points
     vector<Integer> optimal_subdivision_point() const;
@@ -510,7 +532,7 @@ class Matrix {
 
     // try to sort the rows in such a way that the extreme points of the polytope spanned by the rows come first
 
-    size_t extreme_points_first(bool verbose, const vector<Integer> norm = vector<Integer>(0));
+    size_t extreme_points_first(bool verbose, vector<key_t>& perm);
 
     // find an inner point in the cone spanned by the rows of the matrix
 
@@ -536,89 +558,87 @@ class Matrix {
 //                  Matrices of binary expansions
 //---------------------------------------------------------------------------
 
-/* 
+/*
  * Binary matrices contain matrices of nonnegative integers.
  * Each entry is stored "vertically" as the binary expansion of an
- * index (relaive to values) i. The k-th binary digit of i (counting k from 0)
+ * index (relative to values) i. The k-th binary digit of i (counting k from 0)
  * is in layer k.
- * 
+ *
  * The "true" value represented by i is values[i] (or mpz_values[i], see below).
  *
  * The goal is to store large matrices of relatively
  * small numbers with as little space as possible.
- * 
- * Moreover this structure needs as a brifge to nauty.
- * 
+ *
+ * Moreover this structure needs as a bridge to nauty.
+ *
  * It can happen that mpz_class values must be taken into account,
  * even if Integer = long or long long. (See nmz_nauty.cpp,
- * makeMMFromGensOnly). Therefore we have a field mpz_values in 
- * addition to values. Transfer to *this via get_data_mpz. 
+ * makeMMFromGensOnly). Therefore we have a field mpz_values in
+ * addition to values. Transfer to *this via get_data_mpz.
  */
 
 template <typename Integer>
 class BinaryMatrix {
-    
     template <typename>
     friend class BinaryMatrix;
-    
+
     vector<vector<dynamic_bitset> > Layers;
     size_t nr_rows, nr_columns;
     // mpz_class offset;  // to be added to "entries" to get true value
-    
+
     vector<Integer> values;
     vector<mpz_class> mpz_values;
 
-    public:
-       
+   public:
     void insert(long val, key_t i, key_t j);
-    
+
     size_t get_nr_rows() const;
     size_t get_nr_columns() const;
     size_t get_nr_layers() const;
 
     bool test(key_t i, key_t j, key_t k) const;
     long val_entry(size_t i, size_t j) const;
-    Matrix<Integer> get_value_mat() const; 
+    Matrix<Integer> get_value_mat() const;
     Matrix<mpz_class> get_mpz_value_mat() const;
-    
+
     const vector<vector<dynamic_bitset> >& get_layers() const;
     const vector<Integer>& get_values() const;
     const vector<mpz_class>& get_mpz_values() const;
-    
+
     BinaryMatrix();
     BinaryMatrix(size_t m, size_t n);
     BinaryMatrix(size_t m, size_t n, size_t height);
     BinaryMatrix reordered(const vector<key_t>& row_order, const vector<key_t>& col_order) const;
     bool equal(const BinaryMatrix& Comp) const;
-    
-    void get_data_mpz(BinaryMatrix<mpz_class>&  BM_mpz);
+
+    void get_data_mpz(BinaryMatrix<mpz_class>& BM_mpz);
     void set_values(const vector<Integer>& V);
-    
+
     void pretty_print(std::ostream& out, bool with_row_nr = false) const;
 };
 
 template <typename Integer>
-bool BM_compare(const BinaryMatrix<Integer>& A, const BinaryMatrix<Integer>& B){
-    if(A.get_nr_rows() < B.get_nr_rows())
-            return true;
-    if(A.get_nr_rows() > B.get_nr_rows())
-            return false;
-    if(A.get_nr_columns() < B.get_nr_columns())
-            return true;
-    if(A.get_nr_columns() > B.get_nr_columns())
-            return false;
-    if(A.get_values() < B.get_values())
-            return true;
-    if(A.get_values() > B.get_values())
-            return false;
-    if(A.get_mpz_values() < B.get_mpz_values())
-            return true;
-    if(A.get_mpz_values() > B.get_mpz_values())
-            return false;
-     if(A.get_layers() < B.get_layers())
-         return true;
-     return false;
-} 
+bool BM_compare(const BinaryMatrix<Integer>& A, const BinaryMatrix<Integer>& B) {
+    if (A.get_nr_rows() < B.get_nr_rows())
+        return true;
+    if (A.get_nr_rows() > B.get_nr_rows())
+        return false;
+    if (A.get_nr_columns() < B.get_nr_columns())
+        return true;
+    if (A.get_nr_columns() > B.get_nr_columns())
+        return false;
+    if (A.get_values() < B.get_values())
+        return true;
+    if (A.get_values() > B.get_values())
+        return false;
+    if (A.get_mpz_values() < B.get_mpz_values())
+        return true;
+    if (A.get_mpz_values() > B.get_mpz_values())
+        return false;
+    if (A.get_layers() < B.get_layers())
+        return true;
+    return false;
+}
 // class end *****************************************************************
 //                  LLL with returned transformation matrices
 //---------------------------------------------------------------------------
@@ -636,9 +656,11 @@ Matrix<Number> LLL_red(const Matrix<Number>& U, Matrix<Integer>& T, Matrix<Integ
 
     Matrix<Number> Lred = U;
     size_t dim = U.nr_of_columns();
-    int n = U.nr_of_rows();
+    size_t n = U.nr_of_rows();
     // pretty_print(cout);
-    assert((int)U.rank() == n);
+    if(U.rank() != n){
+        throw ArithmeticException();
+    }
     if (n <= 1)
         return Lred;
 
@@ -729,8 +751,9 @@ vector<vector<T> > to_matrix(const vector<T>& v) {
     return mat;
 }
 
-template <typename Integer>
-Matrix<Integer> readMatrix(const string project);
+// For
+// Matrix<Integer> readMatrix(const string project);
+// see inout.h
 
 //---------------------------------------------------------------------------
 //                  Conversion between integer types
@@ -761,8 +784,16 @@ void convert(Matrix<ToType>& to_mat, const Matrix<FromType>& from_mat) {
             convert(to_mat[i][j], from_mat[i][j]);
 }
 
+template<typename Val, typename Ret>
+void convert_matrix_via_string(Matrix<Ret>& V, const Matrix<Val>& W){
+
+    V.resize(W.nr_of_rows(), W.nr_of_columns());
+    for(size_t i = 0; i < W.nr_of_rows(); ++i)
+        convert_vector_via_string(V[i], W[i]);
+}
+
 //---------------------------------------------------------------------------
-//                  Matrix relateed functions
+//                  Matrix related functions
 //---------------------------------------------------------------------------
 // determines the maximal subsets in a vector of subsets given by their indicator vectors
 // result returned in is_max_subset -- must be initialized outside
@@ -771,9 +802,9 @@ void convert(Matrix<ToType>& to_mat, const Matrix<FromType>& from_mat) {
 template <typename IncidenceVector>
 void maximal_subsets(const vector<IncidenceVector>& ind, IncidenceVector& is_max_subset);
 
-// computes the incidence of LinForms withz Gens ijn the following sense:
+// computes the incidence of LinForms with Gens in the following sense:
 // Incidence[i][j] = 1 <==> scalar product(LinForms[i], Gnes[j]) == 0
-template<typename Integer>
+template <typename Integer>
 void makeIncidenceMatrix(vector<dynamic_bitset>& Incidence, const Matrix<Integer>& Gens, const Matrix<Integer>& LinForms);
 
 }  // namespace libnormaliz
